/* initng-conf-gtk, the gtk configuration program for initng
 * Copyright (C) 2006 Daniel Malmgren <dm@mensa.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <pthread.h>
#include <glade/glade.h>
#include <glib/gthread.h>
#include <gconf/gconf-client.h>

#include "rlparser.h"
#include "serviceparser.h"
#include "ngehandler.h"
#include "initng-conf-gtk_globals.h"
#include "gconf.h"

/*
 * Function to setup toolbars, menubars
 * and other misc static stuff.
 */
void misc_config(GtkWidget * main_widget) {

  /* Widgets to add to main_widget */
  GtkWidget *runlevel_filter_combo;
  GtkWidget *zap_button, *edit_button;
  
  /* Get pointers for these widgets */
  runlevel_filter_combo = glade_xml_get_widget(xml, "runlevel_filter_combo");
  zap_button = glade_xml_get_widget(xml, "zap_button");
  edit_button = glade_xml_get_widget(xml, "edit_button");

  /*
   * note:
   * Sensitive widget, meens that users can interact with them.
   */
  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "save_button"), FALSE);
  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "save1"), FALSE);
  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "add_button"), FALSE);
  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "remove_button"), FALSE);
  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "start_button"), FALSE);
  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "stop_button"), FALSE);
  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "restart_button"), FALSE);
  gtk_widget_set_sensitive(zap_button, FALSE);
  gtk_widget_set_sensitive(edit_button, FALSE);

  gtk_combo_box_set_active(GTK_COMBO_BOX(runlevel_filter_combo), 1);

  if(get_gconf_bool(GCONF_SHOW_RUNLEVEL_EDITOR)) {
    GtkWidget *show_runlevel_editor = glade_xml_get_widget(xml,
							   "show_runlevel_editor");
    GtkWidget *runlevel_editor = glade_xml_get_widget(xml, "runlevel_editor");
    GtkWidget *add_button = glade_xml_get_widget(xml, "add_button");
    GtkTreeView *service_list = GTK_TREE_VIEW(glade_xml_get_widget(xml,
								   "service_list"));
    GtkTreeSelection *selected = gtk_tree_view_get_selection(service_list);
    GtkWidget *runlevel_combo = glade_xml_get_widget(xml, "runlevel_combo");
    gtk_combo_box_set_active(GTK_COMBO_BOX(runlevel_combo), 0);
    update_rllist(runlevel_combo);
    if(gtk_tree_selection_get_selected(selected, NULL, NULL))
      gtk_widget_set_sensitive(add_button, TRUE);
    gtk_widget_show_all(runlevel_editor);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(show_runlevel_editor), TRUE);
  }
  if(get_gconf_bool(GCONF_SHOW_ADVANCED)) {
    GtkWidget *show_advanced = glade_xml_get_widget(xml, "show_advanced");
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(show_advanced), TRUE);
  } else {
    gtk_widget_hide(zap_button);
    gtk_widget_hide(edit_button);
  }
  if(get_gconf_bool(GCONF_SHOW_TOOLBAR)) {
    GtkWidget *show_toolbar = glade_xml_get_widget(xml, "show_toolbar");
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(show_toolbar), TRUE);
  } else {
    GtkWidget *toolbar = glade_xml_get_widget(xml, "toolbar");
    gtk_widget_hide(toolbar);
  }
  if(get_gconf_bool(GCONF_SHOW_STATUSBAR)) {
    GtkWidget *show_statusbar = glade_xml_get_widget(xml, "show_statusbar");
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(show_statusbar), TRUE);
  } else {
    GtkWidget *statusbar = glade_xml_get_widget(xml, "statusbar");
    gtk_widget_hide(statusbar);
  }
}


/*
 * Here is MAIN
 */
int main (int argc, char *argv[])
{
  GtkWidget *initng_conf_gtk_app;    /* The main widget  */
  pthread_t ngethread;        /* The nge-thread   */

  /* Enable threading, to make glib and gtk thread safe */
  g_thread_init (NULL);
  gdk_threads_init();

  /* Enable native language support */
#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  /* These enables initng_conf_gtk --help option, an others */
  gtk_init(&argc, &argv);

  glade_init();

  xml = glade_xml_new(FILENAME, ROOTNODE, NULL);

  if(!xml)
    xml = glade_xml_new(SECONDARY_FILENAME, ROOTNODE, NULL);
  
  /* interface.c , This creates the main widget */
  initng_conf_gtk_app = glade_xml_get_widget(xml, "initng_conf_gtk_app");

  /* serviceparser.c , This parsers all .i files, looking for services */
  populate_service_list(initng_conf_gtk_app);
  
  /* rlparser.c , This parses al .runlevel files */
  populate_runlevel_combo(initng_conf_gtk_app);
  
  /* rlparser.c , Initziate the runlevel list widget */  
  init_runlevel_list(initng_conf_gtk_app);
  
  /* from abow, fills toolbars and set up standard stuff */
  misc_config(initng_conf_gtk_app);

  /* Okay, Showtime */
  glade_xml_signal_autoconnect(xml);

  /* Create the thread that will listen to nge, and update status realtime */
  pthread_create(&ngethread, NULL, nge_thread, initng_conf_gtk_app);

  gdk_threads_enter();

  /* go, mainloop */
  gtk_main ();
  
  /* release gtk thread lock */
  gdk_threads_leave();

  /* exit happily */
  return(0);
}

