/* initng-conf-gtk, the gtk configuration program for initng
 * Copyright (C) 2006 Daniel Malmgren <dm@mensa.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>

#include <dirent.h>
#include <fnmatch.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "rlparser.h"
#include "initng-conf-gtk_globals.h"

#define BASEDIR "/etc/initng"
#define RLBASEDIR BASEDIR "/runlevel"

/* Add a new runlevel. Ask the user of the name and create the new (empty)
   entry in the runlevel combo box */
void new_runlevel(const gchar *rl_name) {
  struct rl_struct *this_rl_struct = malloc(sizeof(struct rl_struct));
  GtkWidget *runlevel_combo = glade_xml_get_widget(xml, "runlevel_combo");
  GtkStatusbar *statusbar = GTK_STATUSBAR(glade_xml_get_widget(xml, "statusbar"));
  guint context_id = gtk_statusbar_get_context_id(statusbar, "");
  GtkToggleButton *radio = GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_new_runlevel, 
								"radiobutton_runlevel"));
  
  char *filename = malloc(strlen(rl_name) + 10);
  strcpy(filename, rl_name);
  if(gtk_toggle_button_get_active(radio))
    strcat(filename, ".runlevel");
  else
    strcat(filename, ".virtual");

  gtk_combo_box_prepend_text(GTK_COMBO_BOX(runlevel_combo), filename);
  gtk_combo_box_set_active(GTK_COMBO_BOX(runlevel_combo), 0);

  this_rl_struct->filename = malloc(strlen(RLBASEDIR) + 1 + strlen(filename));
  strcpy(this_rl_struct->filename, RLBASEDIR);
  strcat(this_rl_struct->filename, "/");
  strcat(this_rl_struct->filename, filename);
  
  this_rl_struct->needs_saving = TRUE;
  this_rl_struct->store = gtk_list_store_new (1, G_TYPE_STRING);
  this_rl_struct->next = runlevel_struct;
  runlevel_struct = this_rl_struct;

  update_rllist(runlevel_combo);
  gtk_statusbar_pop(statusbar, context_id);
  gtk_statusbar_push(statusbar, context_id, _("New runlevel created."));

  free(filename);
}

/* Save all changes made to (all) runlevels */
void save_rl() {
  struct rl_struct *this_rl_struct = runlevel_struct;
  GtkWidget *save_button, *save1;
  GtkTreeIter iter;
  gboolean valid;
  FILE *file;

  save_button = glade_xml_get_widget(xml, "save_button");
  save1 = glade_xml_get_widget(xml, "save1");
  while(this_rl_struct != NULL) {
    if(this_rl_struct->needs_saving) {

      file = fopen(this_rl_struct->filename, "w");   /* Open file. */

      if (file == NULL) {
        g_print(_("Unable to open file %s for writing!\n"), this_rl_struct->filename);
        return;
      }

      valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(this_rl_struct->store),
                                            &iter);
      while(valid) {
        gchar *str_data;
        gtk_tree_model_get(GTK_TREE_MODEL(this_rl_struct->store),
                           &iter, 0, &str_data, -1);
        fputs(str_data, file);
        fputs("\n", file);
        g_free(str_data);
        valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(this_rl_struct->store),
                                         &iter);
      }

      fclose(file);
      
      this_rl_struct->needs_saving = FALSE;
    }
    this_rl_struct = this_rl_struct->next;
  }
  gtk_widget_set_sensitive(save_button, FALSE);
  gtk_widget_set_sensitive(save1, FALSE);
}

/* Add the marked service in the service list to the active runlevel */
void add_to_rl(const gchar *name) {
  GtkWidget *runlevel_list;
  GtkTreeIter rliter;
  GtkListStore *rlstore;
  GtkWidget *runlevel_combo;
  gint choosen_rl;
  struct rl_struct *this_rl_struct = runlevel_struct;
  int i;

  runlevel_list = glade_xml_get_widget(xml, "runlevel_list");
  rlstore = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(runlevel_list)));

  if(strrchr(name, '*')) {
    gchar *newname = malloc(strlen(name));
    xml_add_service = glade_xml_new(FILENAME, "add_service", NULL);
    if(!xml_add_service)
      xml_add_service = glade_xml_new(SECONDARY_FILENAME,
				      "add_service", NULL);
    GtkWidget *add_service_name_entry = glade_xml_get_widget(xml_add_service,
							     "add_service_name_entry");
    strncpy(newname, name, strlen(name)-1);
    newname[strlen(name)-1] = 0;
    gtk_entry_set_text(GTK_ENTRY(add_service_name_entry), newname);
    gtk_editable_set_position(GTK_EDITABLE(add_service_name_entry), -1);
    glade_xml_signal_autoconnect(xml_add_service);
    free(newname);
  } else {
    gtk_list_store_append (rlstore, &rliter);
    gtk_list_store_set (rlstore, &rliter, 0, name, -1);

    runlevel_combo = glade_xml_get_widget(xml, "runlevel_combo");  
    choosen_rl = gtk_combo_box_get_active(GTK_COMBO_BOX(runlevel_combo));
    
    for(i = 0; i < choosen_rl; i++) {
      this_rl_struct = this_rl_struct->next;
    }
    this_rl_struct->needs_saving = TRUE;
    GtkWidget *save_button = glade_xml_get_widget(xml, "save_button");
    GtkWidget *save1 = glade_xml_get_widget(xml, "save1");
    gtk_widget_set_sensitive(save_button, TRUE);
    gtk_widget_set_sensitive(save1, TRUE);
    GtkWidget *statusbar = glade_xml_get_widget(xml, "statusbar");
    guint context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "");
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar), context_id);
    char *message = malloc(strlen(name) + 21);
    sprintf(message, _("%s added to runlevel"), name);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), context_id, message);
    free(message);
  }
}

/* Remmove the marked service in the runlevel list from that runlevel */
void remove_from_rl() {
  GtkWidget *runlevel_list;
  GtkTreeIter iter;
  GtkTreeSelection *selected;
  GtkTreeModel *model;
  GtkWidget *runlevel_combo, *save_button, *save1;
  gint choosen_rl;
  struct rl_struct *this_rl_struct = runlevel_struct;
  int i;
  
  runlevel_list = glade_xml_get_widget(xml, "runlevel_list");
  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(runlevel_list));
  if(gtk_tree_selection_get_selected(selected, &model, &iter)) {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    
    save_button = glade_xml_get_widget(xml, "save_button");
    save1 = glade_xml_get_widget(xml, "save1");
    runlevel_combo = glade_xml_get_widget(xml, "runlevel_combo"); 
    choosen_rl = gtk_combo_box_get_active(GTK_COMBO_BOX(runlevel_combo));
    
    for(i = 0; i < choosen_rl; i++) {
      this_rl_struct = this_rl_struct->next;
    }
    this_rl_struct->needs_saving = TRUE;
    gtk_widget_set_sensitive(save_button, TRUE);
    gtk_widget_set_sensitive(save1, TRUE);
  }
}

/* Check which files are runlevels */
int rlfilefilter(const struct dirent* entry) {
  if(!fnmatch("*.runlevel", entry->d_name, 0) ||
     !fnmatch("*.virtual", entry->d_name, 0))
    return 1;
  else
    return 0;
}

/* Update the runlevel list with the runlevel that's active in the
   runlevel combo */
void update_rllist(GtkWidget *runlevel_combo) {
  gint choosen_rl = gtk_combo_box_get_active(GTK_COMBO_BOX(runlevel_combo));
  struct stat buff;
  FILE *file;              /* File descriptor for file */
  GtkTreeIter iter;
  GtkWidget *runlevel_list;
  GtkWidget *remove_button, *add_button;
  int i;
  struct rl_struct *this_rl_struct = runlevel_struct;

  remove_button = glade_xml_get_widget(xml, "remove_button");
  add_button = glade_xml_get_widget(xml, "add_button");
  runlevel_list = glade_xml_get_widget(xml, "runlevel_list");  

  gtk_widget_set_sensitive(remove_button, FALSE);

  for(i = 0; i < choosen_rl; i++) {
    this_rl_struct = this_rl_struct->next;
  }
  
  if(this_rl_struct->store == NULL) {
    this_rl_struct->store = gtk_list_store_new (1, G_TYPE_STRING);

    if(lstat(this_rl_struct->filename, &buff)<0)
      perror(_("Runlevel file doesn't exist!"));
  
    file = fopen(this_rl_struct->filename, "rt");   /* Open file. */

    if (file == NULL) 
      return;

    /* Allocate a file buffer */
    char* buffer = malloc(64); //UGLY!

    /* Read whole file */
    while(fgets(buffer, 64, file) > 0) {
      if(strlen(buffer) > 1) {
        buffer[strlen(buffer)-1] = 0;
        gtk_list_store_append (this_rl_struct->store, &iter);
        gtk_list_store_set (this_rl_struct->store, &iter, 0, buffer, -1);
      }
    }

    fclose(file);
    free(buffer);  
  }
  gtk_tree_view_set_model(GTK_TREE_VIEW(runlevel_list),
                          GTK_TREE_MODEL(this_rl_struct->store));
}

/* Populate the runlevel combo with all runlevels found */
void populate_runlevel_combo() {
  GtkWidget *runlevel_combo;
  runlevel_struct = malloc(sizeof(struct rl_struct));
  struct rl_struct *this_rl_struct = runlevel_struct;
  int i;

  runlevel_combo = glade_xml_get_widget(xml, "runlevel_combo");

  struct dirent **namelist;
  int n = scandir(RLBASEDIR, &namelist, &rlfilefilter, alphasort);
  if(n < 1) {
    perror(_("Unable to read any runlevel files"));
    exit(EXIT_FAILURE);
  }

  //Don't ask me about this line. The only way I got this box to work
  //was by adding an empty line to it using Glade. I then remove that
  //line here. Nicer solutions are wanted!
  gtk_combo_box_remove_text(GTK_COMBO_BOX(runlevel_combo), 0);
  for(i = 0; i < n; i++) {
    this_rl_struct->filename = malloc(strlen(RLBASEDIR) + 2 +
                                      strlen(namelist[i]->d_name));
    strcpy(this_rl_struct->filename, RLBASEDIR);
    strcat(this_rl_struct->filename, "/");
    strcat(this_rl_struct->filename, namelist[i]->d_name);
    
    gtk_combo_box_append_text(GTK_COMBO_BOX(runlevel_combo),
			      namelist[i]->d_name);
    free(namelist[i]);
    this_rl_struct->needs_saving = FALSE;
    this_rl_struct->store = NULL;
    if(i < n-1) {
      this_rl_struct->next = malloc(sizeof(struct rl_struct));
      this_rl_struct = this_rl_struct->next;
    } else {
      this_rl_struct->next = NULL;
    }
  }
  free(namelist);
}

void init_runlevel_list() {
  GtkWidget *runlevel_list;
  GtkCellRenderer *rlrenderer;

  rlrenderer = gtk_cell_renderer_text_new();
  runlevel_list = glade_xml_get_widget(xml, "runlevel_list");
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(runlevel_list),
                                              -1, _("Name"),
					      rlrenderer, "text", 0,
					      NULL);
}
